import Tkinter
import sys

class Control(Tkinter.Frame):
    def __init__(self, master, integer=1, step=1, value=0.0, min=-sys.maxint, max=sys.maxint,\
                 allowempty=0, label='', labelside='w', variable=None, command=None, **kw):
        Tkinter.Frame.__init__(self, master, **kw)
        
        self._integer = integer
        self._step = step
        self._max = max
        self._min = min
        self._allowempty = allowempty
        self._command = command
        
        if not variable:
            if integer:
                self._var = Tkinter.IntVar()
            else:
                self._var = Tkinter.DoubleVar()
        else:
            self._var = variable
        self._textvar = Tkinter.StringVar()
        self.set(value)

        grid_opts = {'w':(0, 0, 0, 1), 'e':(0, 1, 0, 0), 'n':(0, 0, 1, 0), 's':(1, 0, 0, 0)}
        
	self._arrow_up = Tkinter.PhotoImage(data='R0lGODlhBwAEAPABAAAAAP///yH5BAEAAAEALAAAAAAHAAQAAAIRTBgwYcKAABMGBAgwIECAQAEAOw==')
	self._arrow_down = Tkinter.PhotoImage(data='R0lGODlhBwAEAPABAAAAAP///yH5BAEAAAEALAAAAAAHAAQAAAIRBAgQIEKAABEmBIgwYUKESQEAOw==')
        
        self.label = Tkinter.Label(self, text=label)
        self.label.grid(row=grid_opts[labelside][0], column=grid_opts[labelside][1])

        self._frame = Tkinter.Frame(self, relief='groove', bd=2)
        self._frame.grid(row=grid_opts[labelside][2], column=grid_opts[labelside][3])
        self._frame.grid_rowconfigure(0, weight=1)
        self.entry = Tkinter.Entry(self._frame, width=4, textvariable=self._textvar)
        self.entry.grid(row=0, column=0)
        self.entry.bind('<Any-KeyRelease>', self._apply_value)
        self.entry.bind('<FocusOut>', self.set)
        self.entry.bind('<Return>', self._return_callback)
        self.entry.bind('<Up>', self._increase)
        self.entry.bind('<Down>', self._decrease)
        
        self._bframe = Tkinter.Frame(self._frame)
        self._bframe.grid(row=0, column=1, sticky='ns')
        self.up = Tkinter.Button(self._bframe, image=self._arrow_up, pady=0, padx=0,\
                         relief='flat', bd=1, takefocus=0)
        self.up.grid(row=0, column=0, sticky='n')
        self.down = Tkinter.Button(self._bframe, image=self._arrow_down, pady=0, padx=0,\
                           relief='flat', bd=1, takefocus=0)
        self.down.grid(row=1, column=0, sticky='s')

        #some stuff we need for repeated in-/decreasing the value while B1 is pressed
        self._mousepress = 0
        for button in (self.up, self.down):
            button.bind('<1>', self._start_buttons)
            button.bind('<Any-ButtonRelease-1>', self._stop_buttons)
            button.bind('<Leave>', self._stop_buttons)
                

    def get(self):
        return self._var.get()
       
    def set(self, event=None, value=None):
        if not (self._allowempty and value == None and not self._textvar.get()):
            if value == None:
                value = self._check_value()
            if self._integer:
                value = int(value)
            if value > self._max or value < self._min:
                value = self._var.get()
            self._var.set(value)
            self._textvar.set(str(value))
            if self._command:
                self._command()

    def disable(self):
        self.up.configure(state='disabled')
        self.down.configure(state='disabled')
        self.entry.selection_clear()
        self.entry.configure(state='disabled', fg='gray')

    def enable(self):
        self.up.configure(state='normal')
        self.down.configure(state='normal')
        self.entry.configure(state='normal', fg='black')
                            
    def _check_value(self, event=None):
        try:
            if self._integer:
               self._var.set(int(self._textvar.get()))
            else:
                self._var.set(float(self._textvar.get()))
        except:
            self._textvar.set(str(self._var.get()))
        return self._var.get()

    def _apply_value(self, event):
        try:
            if self._integer:
                self._var.set(int(self._textvar.get()))
            else:
                self._var.set(float(self._textvar.get()))
            if self._command:
                self._command()
        except:
            pass
    
    def _increase(self, event=None):
        if self.entry['state'] != 'disabled':
            old = self._var.get()
            new = old + self._step
            self.set(value=new)
    
    def _decrease(self, event=None):
        if self.entry['state'] != 'disabled':
            old = self._var.get()
            new = old - self._step
            self.set(value=new)

    def _button_increase(self):
        if self._mousepress:
            self._increase()
            self.after(50, self._button_increase)
            
    def _button_decrease(self):
        if self._mousepress:
            self._decrease()
            self.after(50, self._button_decrease)

    def _start_buttons(self, event):
        if self.entry['state'] != 'disabled':
            self.entry.focus_set()
            self._mousepress = 1
            if event.widget == self.up:
                self._increase()
                self.after(500, self._button_increase)
            elif event.widget == self.down:
                self._decrease()
                self.after(500, self._button_decrease)

    def _stop_buttons(self, event):
        self._mousepress = 0
        
    def _return_callback(self, event=None):
        self.entry.event_generate('<Tab>')
