#!/usr/bin/env python

# scsiaddgui - a gui for the scsiadd utility
# copyright (c) 2003 - 2004 Michael Lange <klappnase@8ung.at>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the #hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from Tkinter import *
from Control import Control
from ScrolledText import ScrolledText
import sys
import tkMessageBox
import commands
import fileinput
import os
#check the language:
from messages_en import *
helpfile = os.path.join(sys.path[0], 'help_en')

lang = rclang = ''
try:
    lang = os.environ['LANG']
except:
    pass
#SuSE special:
try:
    rclang = os.environ['RC_LANG']
except:
    pass

de = ('de', 'De', 'DE')
if lang[0:2] in de or rclang[0:2] in de:
    from messages_de import *
    helpfile = os.path.join(sys.path[0], 'help_de')

option_DB = sys.path[0] + '/optionDB'

class NiceButton(Frame):
    '''A Tkinter.Button widget with a nicer look and feel.'''
    def __init__(self, master, width=16, relief='groove', bd=3, border_relief='solid',\
                 border_bd=1, highlightthickness=1, **kw):
        #a nice black border:
        Frame.__init__(self, master, relief=border_relief, bd=border_bd)
        self.button = Button(self, relief=relief, bd=bd, width=width,\
                             highlightthickness=highlightthickness, **kw)
        #we want the button to invoke its callback on <Return> events just like on <space> events:
        self.button.bind('<Return>', lambda event, key='<space>' : self.button.event_generate(key))
        self.button.pack()

class HelpDialog(Toplevel):
    def __init__(self, master, **kw):
        Toplevel.__init__(self, master, **kw)
        self.protocol('WM_DELETE_WINDOW', self.close)
        self.grid_rowconfigure(0, weight=1)
        self.grid_columnconfigure(0, weight=1)
        self.master = master
        
        self.textframe = Frame(self, relief='solid', bd=1)
        self.textframe.grid(row=0, column=0, sticky='news', padx=3, pady=3)
        self.text = ScrolledText(self.textframe, wrap='word')
        self.text.pack(fill='both', expand=1)
        self.text.configure(bg="wheat", selectbackground="wheat", selectforeground="black",\
                            selectborderwidth=0, cursor='', padx=4, pady=4)
        for line in fileinput.input(helpfile):
            self.text.insert('end', line)
        self.text.config(state='disabled')
        self.closeb = NiceButton(self, text=helpbuttonmsg, command=self.close)
        self.closeb.grid(row=1, column=0, pady=5)

    def close(self, event=None):
        self.master.help_dialog = None
        self.destroy()

class OptConf:
    def __init__(self):
        self.opts = {'host':0, 'chan':0, 'i_d':0, 'lun':0}
        self.conffile = os.path.join(os.getenv('HOME'), '.scsiaddgui')

    def read_conffile(self):
        try:
            for line in fileinput.input(self.conffile):
                try:
                    l = line.split('=')
                    if self.opts.has_key(l[0]):
                        self.opts[l[0]] = int(l[1])
                except:
                    pass
        except:
            pass

    def write_conffile(self):
        f = open(self.conffile, 'w')
        for key in self.opts:
            f.write(key + '=' + str(self.opts[key]) + '\n')
        f.close()

        
class Mainwindow(Frame):
    def __init__(self, master, **kw):
        Frame.__init__(self, master, **kw)
        self.grid_rowconfigure(0, weight=1)
        for i in range(0, 4):
            self.grid_columnconfigure(i, weight=1)

        self.master = master
        self.help_dialog = None
        self.conf = OptConf()
        self.conf.read_conffile()
        
        self.host = IntVar()
        self.host.set(self.conf.opts['host'])
        self.chan = IntVar()
        self.chan.set(self.conf.opts['chan'])
        self.i_d = IntVar()
        self.i_d.set(self.conf.opts['i_d'])
        self.lun = IntVar()
        self.lun.set(self.conf.opts['lun'])
        self.devices = ''
        
        self.textframe = Frame(self, relief='solid', bd=1)
        self.textframe.grid(row=0, column=0, columnspan=4, sticky='news', pady=3, padx=3)
        self.devicelist = ScrolledText(self.textframe)
        self.devicelist.pack(fill='both', expand=1)
        self.devicelist.configure(selectbackground='white', selectforeground='black', takefocus=0, width=50, height=15,\
                                        selectborderwidth=0, padx=4, pady=4, cursor='', state='disabled')
        self.hostselect = Control(self, min=0, max=7, label=hostmsg, variable=self.host)
        self.hostselect.grid(row=1, column=0)
        self.chanselect = Control(self, min=0, max=7, label=chanmsg, variable=self.chan)
        self.chanselect.grid(row=1, column=1)
        self.idselect = Control(self, min=0, max=15, label=idmsg, variable=self.i_d)
        self.idselect.grid(row=1, column=2)
        self.lunselect = Control(self, min=0, max=255, label=lunmsg, variable=self.lun)
        self.lunselect.grid(row=1, column=3)
        
        self.bframe = Frame(self)
        self.bframe.grid(row=2, column=0, columnspan=4, pady=8, sticky='ew')
        for c in range(3):
            self.bframe.grid_columnconfigure(c, weight=1)
        self.scanbutton = NiceButton(self.bframe, text=scanbuttonmsg, command=self.scanbus)
        self.scanbutton.grid(row=0, column=0, padx=8)
        self.addbutton = NiceButton(self.bframe, text=addbuttonmsg, command=self.adddevice)
        self.addbutton.grid(row=0, column=1, padx=8)
        self.removebutton = NiceButton(self.bframe, text=removebuttonmsg, command=self.removedevice)
        self.removebutton.grid(row=0, column=2, padx=8)
    
    def sysadmin(self):
        sa = os.getuid()
        if not sa:
            return 1
        else:
            tkMessageBox.showerror(title='scsiaddgui', message=nopermissionmsg)
            return 0
    
    def scanbus(self, event=None):
        if self.sysadmin():
            self.master.configure(cursor='watch')
            self.master.update_idletasks()
            cmd = ' '.join(('scsiadd', '-s', str(self.host.get()), str(self.chan.get())))
            self.exec_cmd(cmd)
            self.master.configure(cursor='')

    def adddevice(self, event=None):
        if self.sysadmin():
            cmd =' '.join(('scsiadd', '-a', str(self.host.get()), str(self.chan.get()),\
                           str(self.i_d.get()), str(self.lun.get())))
            self.exec_cmd(cmd)
    
    def removedevice(self, event=None):
        if self.sysadmin():
            cmd = ' '.join(('scsiadd', '-r', str(self.host.get()), str(self.chan.get()),\
                           str(self.i_d.get()), str(self.lun.get())))
            self.exec_cmd(cmd)
        
    def getstatus(self):
        cmd = 'cat /proc/scsi/scsi'
        self.exec_cmd(cmd)
    
    def exec_cmd(self, cmd):
        self.devices = commands.getoutput(cmd)
        self.updatedevicelist()
        
    def updatedevicelist(self):
        d = self.devices
        # ban this ugly message frome the screen : "scsiadd:scsiadd():could not close /proc/scsi/scsi (w): Function not implemented"
        # that appears each time scsiadd -s is called; maybe even other error messages will be kept away from the screen:
        if d[0] != 'A':
            try:
                d = d.replace(d.split('A')[0], '')
            except:
                pass
        try:
            d = d.replace('Attached devices', attacheddevmsg)
            d = d.replace('Vendor', vendormsg)
            d = d.replace('Model', modelmsg)
            d = d.replace('Type', typemsg)
            d = d.replace('Host:', '\n' + hostmsg)
            d = d.replace('Channel:', chanmsg)
            d = d.replace('Id:', idmsg)
            d = d.replace('Lun:', lunmsg)
        except:
            pass
        self.devicelist['state'] = 'normal'
        self.devicelist.delete(0.0, 'end')
        self.devicelist.insert('end', d)
        self.devicelist['state'] = 'disabled'

    def show_help(self):
        if self.help_dialog:
            self.help_dialog.tkraise()
            self.help_dialog.deiconify()
        else:
            self.help_dialog = HelpDialog(self, relief='ridge', bd=3)

    def close(self):
        self.conf.opts['host'] = self.host.get()
        self.conf.opts['chan'] = self.chan.get()
        self.conf.opts['i_d'] = self.i_d.get()
        self.conf.opts['lun'] = self.lun.get()
        self.conf.write_conffile()
        self.master.quit()
    

def infobox():
    tkMessageBox.showinfo(title='scsiaddgui - Info',\
                          message="ScsiaddGui - by Michael Lange\nContact: <klappnase@8ung.at>")

def main():
    root = Tk()
    root.option_readfile(option_DB)
    root.title(string='scsiaddgui')
    mw = Mainwindow(root)
    mw.pack(fill='both', expand=1)
    mw.getstatus()
    root.protocol('WM_DELETE_WINDOW', mw.close)
    menubar = Menu(root)
    progm = Menu(menubar)
    progm.bind('<Key-Alt-' + progmsg[0] + '>')
    progm.add_separator()
    progm.add_command(label=quitmsg, command=mw.close)
    menubar.add_cascade(label=progmsg, underline=0, menu=progm)
    helpm = Menu(menubar)
    helpm.bind('<Key-Alt-' + helpmsg[0] + '>')
    helpm.add_separator()
    helpm.add_command(label=helpmsg, command=mw.show_help)
    helpm.add_command(label=infomsg, command=infobox)
    menubar.add_cascade(label=helpmsg, underline=0, menu=helpm)
    root.config(menu=menubar)
    root.mainloop()
    root.destroy()
        
if __name__ == "__main__":
    main()
